
#include "atomGrid.h"


/**************
 *            *
 *  AtomNode  *
 *            *
 **************/

// Constructor
//
AtomNode::AtomNode(Atom* a) :
  atom(a), next(0) {

  return;
}


// Destructor
//
AtomNode::~AtomNode() {

  delete next;

  return;
}


// getNext
//
AtomNode* AtomNode::getNext() {

  return next;
}


// getAtom
//
Atom* AtomNode::getAtom() {

  return atom;
}


// setNext
//
int AtomNode::setNext(AtomNode* node) {

  next = node;

  return 1;
}


// printAtomNode
//
int AtomNode::printAtomNode(FILE* outfile) {

  Coordinate3D* coord = atom->getCoordinate3D();

  if (coord == 0) {
    fprintf(outfile,"    AtomNode: 0\n");
    return 1;
  }
  fprintf(outfile,"    AtomNode: (%3.3f,%3.3f,%3.3f)\n",coord->getX(),coord->getY(),coord->getZ());

  if (next == 0) {
    fprintf(outfile,"    AtomNode: 0\n");
    return 1;
  }
  next->printAtomNode(outfile);

  return 1;
}



/**************
 *            *
 *  AtomList  *
 *            *
 **************/

// Constructor
//
AtomList::AtomList() :
  head(0), current(0) {

  return;
}


// Destructor
//
AtomList::~AtomList() {

  delete head;

  return;
}


// getFirstAtom
//
Atom* AtomList::getFirstAtom() {

  //printf("=>AtomList::getFirstAtom\n");

  current = head;

  if (current == 0) {
    //printf("  current == 0\n");
    //printf("<=AtomList::getFirstAtom: 0\n");
    return 0;
  }

  //printf("  current != 0\n");
  //printf("<=AtomList::getFirstAtom\n");
  return current->getAtom();
}


// getNextAtom
//
Atom* AtomList::getNextAtom() {

  //printf("=>AtomList::getNextAtom\n");

  if (current == 0) {
    //printf("<=AtomList::getNextAtom: 0\n");
    return 0;
  }

  current = current->getNext();

  if (current == 0) {
    //printf("<=AtomList::getNextAtom: 0\n");
    return 0;
  }

  //printf("<=AtomList::getNextAtom\n");
  return current->getAtom();
}


// addAtom()
//
int AtomList::addAtom(Atom* atom) {

  current = head;
  
  if (current == 0) {
    head = new AtomNode(atom);
  }
  else {
    while (current->getNext() != 0) {
      current = current->getNext();
    }
    current->setNext(new AtomNode(atom));
  }

  current = head;

  return 1;
}


// printAtomList
//
int AtomList::printAtomList(FILE* outfile) {

  fprintf(outfile,"  AtomList {\n");
  head->printAtomNode(outfile);
  fprintf(outfile,"  }\n");

  return 1;
}



/******************
 *                *
 *  AtomGridCell  *
 *                *
 ******************/

// Constructor
//
AtomGridCell::AtomGridCell(int a, int b, int c) :
  x(a), y(b), z(c) {

  return;
}


// Destructor
//
AtomGridCell::~AtomGridCell() {

  return;
}


// getX
//
int AtomGridCell::getX() {

  return x;
}


// getY
//
int AtomGridCell::getY() {

  return y;
}


// getZ
//
int AtomGridCell::getZ() {

  return z;
}



/**************
 *            *
 *  AtomGrid  *
 *            *
 **************/

// Constructor
//
AtomGrid::AtomGrid(Box* box) :
  focusIndex(0), distanceUnit(4.0) /*distanceUnit(3.0)*/ {

  //printf("=>AtomGrid\n");

  origin = box->getOrigin();
  width  = (int) ceil(box->getWidth()  / distanceUnit);
  height = (int) ceil(box->getHeight() / distanceUnit);
  depth  = (int) ceil(box->getDepth()  / distanceUnit);

  divide();

  for (int i=0; i<8; i++) {
    focusCells[i] = 0;
  }

  /*
  printf("  width:  %d\n",width);
  printf("  height: %d\n",height);
  printf("  depth:  %d\n",depth);
  
  for (int i=0; i<width; i++) {
    for (int j=0; j<height; j++) {
      for (int k=0; k<depth; k++) {
	if (grid[i][j][k] == 0) {
	  //printf("  La\n");
	  printf("  grid[%d][%d][%d] == 0\n",i,j,k);
	}
      }
    }
  }

  printf("<=AtomGrid\n");
  */
  return;
}


// Destructor
//
AtomGrid::~AtomGrid() {

  for (int i=0; i<width; i++) {
    for (int j=0; j<height; j++) {
      for (int k=0; k<depth; k++) {
	delete grid[i][j][k];
      }
      delete grid[i][j];
    }
    delete grid[i];
  }
  delete grid;

  for (int i=0; i<8; i++) {
    delete focusCells[i];
  }
  delete focusCells;

  return;
}


// getFirstAtom
//   Get first atom in the grid element that encloses
//   the given coordinate
Atom* AtomGrid::getFirstAtom(Coordinate3D* coord) {

  //printf("=>AtomGrid::getFirstAtom\n");

  focusIndex = 0;

  // Clear out previous focusCells
  for (int i=0; i<8; i++) {
    if (focusCells[i] != 0) {
      delete focusCells[i];
      focusCells[i] = 0;
    }
  }

  focusCells[0] = coordinate3DToCell(coord);
  if (focusCells[0] == 0) {
    return 0;
  }

  int x = 0;
  int y = 0;
  int z = 0;
  int numOverlappedSides = 0;

  if (focusCells[0]->getX() < width-1) {
    if (overlapsSide(coord, XP) == 1) {
      x = 1;
      numOverlappedSides++;
    }
  }
  if (focusCells[0]->getY() < height-1) {
    if (overlapsSide(coord, YP) == 1) {
      y = 1;
      numOverlappedSides++;
    }
  }
  if (focusCells[0]->getZ() < depth-1) {
    if (overlapsSide(coord, ZP) == 1) {
      z = 1;
      numOverlappedSides++;
    }
  }
  if (focusCells[0]->getX() > 0) {
    if (overlapsSide(coord, XN) == 1) {
      x = -1;
      numOverlappedSides++;
    }
  }
  if (focusCells[0]->getY() > 0) {
    if (overlapsSide(coord, YN) == 1) {
      y = -1;
      numOverlappedSides++;
    }
  }
  if (focusCells[0]->getZ() > 0) {
    if (overlapsSide(coord, ZN) == 1) {
      z = -1;
      numOverlappedSides++;
    }
  }

  if (numOverlappedSides > 0) {
    //printf("   YOYOYO!!! -- %d\n", numOverlappedSides);
  }

  if (numOverlappedSides > 3) {
    // XXX - ERROR!!!  SHOULD BE NO MORE THAN 3 OVERLAPPED SIDES
    printf("AtomGrid::getFirstAtom - Error) more than 3 overlapped sides\n");
    // XXX - DEBUGGING VARIABLES
    int xn, xp, yn, yp, zn, zp;
    xn = overlapsSide(coord, XN);
    xp = overlapsSide(coord, XP);
    yn = overlapsSide(coord, YN);
    yp = overlapsSide(coord, YP);
    zn = overlapsSide(coord, ZN);
    zp = overlapsSide(coord, ZP);
    printf("  (xn,xp,yn,yp,zn,zp) = (%d,%d,%d,%d,%d,%d)\n",xn,xp,yn,yp,zn,zp);
    printf("  coord: (%3.3f,%3.3f,%3.3f)\n",coord->getX(),coord->getY(),coord->getZ());
    printf("  (width,height,depth):  (%d,%d,%d)\n",width,height,depth);
    printf("  origin: (%3.3f,%3.3f,%3.3f)\n",origin->getX(),origin->getY(),origin->getZ());
  }
  if (numOverlappedSides == 0) {
    AtomList* current = getAtomList(focusCells[0]);
    if (current == 0) {
      //printf("<=AtomGrid::getFirstAtom: 0\n");
      return 0;
    }
    //printf("  focusIndex = %d\n",focusIndex);
    //printf("<=AtomGrid::getFirstAtom: 1\n");
    return current->getFirstAtom();
  }

  setFocusCells(numOverlappedSides,x,y,z);

  for (; focusIndex<8; focusIndex++) {
    //printf("  focusIndex = %d\n",focusIndex);
    AtomList* current = getAtomList(focusCells[focusIndex]);
    if (current != 0) {
      Atom* atom = current->getFirstAtom();
      if (atom != 0) {
	//printf("  focusIndex = %d\n",focusIndex);
	//printf("<=AtomGrid::getFirstAtom: 2\n");
	return atom;
      }
    }
  }

  //printf("<=AtomGrid::getFirstAtom: 0\n");
  return 0;
}


// getNextAtom
//   Get next atom in the grid element that encloses
//   the given coordinate
Atom* AtomGrid::getNextAtom() {

  //printf("=>AtomGrid::getNextAtom\n");
  
  //printf("  focusIndex = %d\n",focusIndex);
  if (focusIndex >= 8) {
    //printf("<=AtomGrid::getNextAtom: 0\n");
    return 0;
  }

  AtomList* current = getAtomList(focusCells[focusIndex]);
  //printf("  Hey1\n");
  if (current != 0) {
    Atom* atom = current->getNextAtom();
    if (atom != 0) {
      //printf("<=AtomGrid::getNextAtom\n");
      return atom;
    }
  }
  focusIndex++;

  //printf("  Hey2\n");

  Atom* atom = 0;
  for (; focusIndex<8; focusIndex++) {
    //printf("  focusIndex = %d\n",focusIndex);
    current = getAtomList(focusCells[focusIndex]);
    if (current != 0) {
      atom = current->getFirstAtom();
      if (atom != 0) {
	//printf("  focusIndex = %d\n",focusIndex);
	//printf("<=AtomGrid::getNextAtom\n");
	return atom;
      }
    }
  }

  //printf("<=AtomGrid::getNextAtom: 0\n");
  return 0;
}


// addAtom
//
int AtomGrid::addAtom(Atom* atom) {

  //printf("=>AtomGrid::addAtom\n");

  /*
  for (int i=0; i<width; i++) {
  for (int j=0; j<height; j++) {
  for (int k=0; k<depth; k++) {
  //printf("  La\n");
  if (grid[i][j][k] == 0) {
  printf("  grid[%d][%d][%d] == 0\n",i,j,k);
  }
  else {
  printf("  grid[%d][%d][%d] != 0\n",i,j,k);
  }
  }
  }
  }
  */

  AtomGridCell* cell = coordinate3DToCell(atom->getCoordinate3D());
  if (cell == 0) {
    delete cell;
    return 0;
  }
  //printf("  Hey\n");
  AtomList* current = getAtomList(cell);
  //printf("<=AtomGrid::getAtomList\n");

  if (current == 0) {
    //printf("  current == 0\n");
    current = new AtomList();
    setAtomList(cell, current);
  }

  current->addAtom(atom);
  //printf("  Hey\n");
  
  delete cell;

  //printf("<=AtomGrid::addAtom\n");

  return 1;
}


// getFocusIndex
//   DEBUGGING METHOD
int AtomGrid::getFocusIndex() {

  return focusIndex;
}


// printAtomGrid
//
int AtomGrid::printAtomGrid(FILE* outfile) {

  fprintf(outfile,"AtomGrid {\n");

  for (int i=0; i<width; i++) {
    for (int j=0; j<height; j++) {
      for (int k=0; k<depth; k++) {
	if (grid[i][j][k] == 0) {
	  printf("  grid[%d][%d][%d]: 0\n",i,j,k);
	}
	else {
	  printf("  grid[%d][%d][%d]:\n",i,j,k);
	  grid[i][j][k]->printAtomList(outfile);
	}
      }
    }
  }
  fprintf(outfile,"}\n");
  
  return 1;
}


// getAtomList
//
AtomList* AtomGrid::getAtomList(AtomGridCell* cell) {
  
  //printf("=>AtomGrid::getAtomList\n");
  /*
    printf("    cell->getX(): %d\n", cell->getX());
    printf("    cell->getY(): %d\n", cell->getY());
    printf("    cell->getZ(): %d\n", cell->getZ());
    
    for (int i=0; i<width; i++) {
    for (int j=0; j<height; j++) {
    for (int k=0; k<depth; k++) {
    if (grid[i][j][k] == 0) {
    printf("  grid[%d][%d][%d] == 0\n",i,j,k);
    }
    }
    }
    }
    
    //AtomList* aList = grid[cell->getX()][cell->getY()][cell->getZ()];
    AtomList* aList = grid[0][0][0];
    
    if (aList == 0) {
    printf("  Hey1\n");
    }
    printf("  Yo1\n");
  */
  
  if (cell == 0) {
    return 0;
  }
  
  //printf("    grid[%d][%d][%d] == 0\n",cell->getX(),cell->getY(),cell->getZ());

  return grid[cell->getX()][cell->getY()][cell->getZ()];
}


// setAtomList
//
int AtomGrid::setAtomList(AtomGridCell* cell, AtomList* list) {

  //printf("=>AtomGrid::setAtomList\n");

  if (getAtomList(cell) != 0) {
    //printf("<=AtomGrid::setAtomList: 0\n");
    return 0;
  }

  grid[cell->getX()][cell->getY()][cell->getZ()] = list;

  //printf("<=AtomGrid::setAtomList\n");
  return 1;
}


// divide
//   Divides the box volume into discrete grid elements
//   that encompass atoms.  Creates the main grid array.
int AtomGrid::divide() {

  //printf("=>AtomGrid::divide\n");

  grid = new AtomList***[width];
  
  for (int i=0; i<width; i++) {
    grid[i] = new AtomList**[height];
    for (int j=0; j<height; j++) {
      grid[i][j] = new AtomList*[depth];
      for (int k=0; k<depth; k++) {
	grid[i][j][k] = 0;
	/*
	  if (grid[i][j][k] == 0) {
	  printf("  grid[%d][%d][%d]\n",i,j,k);
	  }
	*/
      }
    }
  }
  
  //printf("<=AtomGrid::divide\n");

  return 1;
}


// coordinate3DToCell
//
AtomGridCell* AtomGrid::coordinate3DToCell(Coordinate3D* coord) {

  int x = (int) floor( (coord->getX() - origin->getX()) / distanceUnit );
  int y = (int) floor( (coord->getY() - origin->getY()) / distanceUnit );
  int z = (int) floor( (coord->getZ() - origin->getZ()) / distanceUnit );

  if ( x < 0 || x >= width  ||
       y < 0 || y >= height ||
       z < 0 || z >= depth) {
    return 0;
  }

  return new AtomGridCell(x,y,z);
}


// overlapsSide
//
int AtomGrid::overlapsSide(Coordinate3D* coord, int side) {

  if (coord == 0) {
    return 0;
  }

  float x = 0.0;
  float y = 0.0;
  float z = 0.0;

  switch (side) {
  case XP:
    x = coord->getX() - origin->getX();
    x = (ceil(x / distanceUnit) * distanceUnit) - x;
    //if (x < 2.0) {
    if (x < 2.0 && x > 0.0) {
      return 1;
    }
    return 0;
  case XN:
    x = coord->getX() - origin->getX();
    x -= floor(x / distanceUnit) * distanceUnit;
    if (x < 2.0) {
      return 1;
    }
    return 0;
  case YP:
    y = coord->getY() - origin->getY();
    y = (ceil(y / distanceUnit) * distanceUnit) - y;
    //if (y < 2.0) {
    if (y < 2.0 && y > 0.0) {
      return 1;
    }
    return 0;
  case YN:
    y = coord->getY() - origin->getY();
    y -= floor(y / distanceUnit) * distanceUnit;
    if (y < 2.0) {
      return 1;
    }
    return 0;
  case ZP:
    z = coord->getZ() - origin->getZ();
    z = (ceil(z / distanceUnit) * distanceUnit) - z;
    //if (z < 2.0) {
    if (z < 2.0 && z > 0.0) {
      return 1;
    }
    return 0;
  case ZN:
    z = coord->getZ() - origin->getZ();
    z -= floor(z / distanceUnit) * distanceUnit;
    if (z < 2.0) {
      return 1;
    }
    return 0;
  }

  return 0;
}


// setFocusCells
//
int AtomGrid::setFocusCells(int numSides, int x, int y, int z) {

  /**
     if (x == 0) {
     if (y == 0) {
     if (z == 0) {
     return 1;
     }
     // z != 0
     else {
     if (z < 0) {
     
     }
     if (z > 0) {
     
     }
     }
     }
     // y != 0
     else {
     if (y < 0) {
     if (z == 0) {
     return 1;
     }
     // y != 0, z != 0
     else {
     
     }
     }
     if (y > 0) {
     if (z == 0) {
     return 1;
     }
     // y != 0, z != 0
     else {
     if (z < 0) {
     
     }
     if (z > 0) {
     
     }
     }
     }
     }
     }
     // x != 0
     else {
     if (y == 0) {
     if (z == 0) {
     return 1;
     }
     // x != 0, z != 0
     else {
     
     }
     }
     // x != 0, y != 0
     else {
     if (z == 0) {
     return 1;
     }
     // x != 0, y != 0, z != 0
     else {
     
     }
     }
     }
  */
  
  AtomGridCell* focus = focusCells[0];
  int focusX = focus->getX();
  int focusY = focus->getY();
  int focusZ = focus->getZ();
  
  switch (numSides) {
  case 0:
    return 1;
  case 1:
    // x != 0
    if (x < 0) {
      focusCells[1] = new AtomGridCell(focusX-1, focusY, focusZ);
    }
    else if (x > 0) {
      focusCells[1] = new AtomGridCell(focusX+1, focusY, focusZ);
    }
    
    // y != 0
    else if (y < 0) {
      focusCells[1] = new AtomGridCell(focusX, focusY-1, focusZ);
    }
    else if (y > 0) {
      focusCells[1] = new AtomGridCell(focusX, focusY+1, focusZ);
    }
    
    // z != 0
    else if (z < 0) {
      focusCells[1] = new AtomGridCell(focusX, focusY, focusZ-1);
    }
    else if (z > 0) {
      focusCells[1] = new AtomGridCell(focusX, focusY, focusZ+1);
    }
    return 1;

  case 2:
    if (x == 0) {
      if (y < 0) {
	focusCells[1] = new AtomGridCell(focusX, focusY-1, focusZ);
	if (z < 0) {
	  focusCells[2] = new AtomGridCell(focusX, focusY,   focusZ-1);
	  focusCells[3] = new AtomGridCell(focusX, focusY-1, focusZ-1);
	}
	else if (z > 0) {
	  focusCells[2] = new AtomGridCell(focusX, focusY,   focusZ+1);
	  focusCells[3] = new AtomGridCell(focusX, focusY-1, focusZ+1);
	}
      }
      else if (y > 0) {
	focusCells[1] = new AtomGridCell(focusX, focusY+1, focusZ);
	if (z < 0) {
	  focusCells[2] = new AtomGridCell(focusX, focusY,   focusZ-1);
	  focusCells[3] = new AtomGridCell(focusX, focusY+1, focusZ-1);
	}
	else if (z > 0) {
	  focusCells[2] = new AtomGridCell(focusX, focusY,   focusZ+1);
	  focusCells[3] = new AtomGridCell(focusX, focusY+1, focusZ+1);
	}
      }
    }
    
    else if (y == 0) {
      if (x < 0) {
	focusCells[1] = new AtomGridCell(focusX-1, focusY, focusZ);
	if (z < 0) {
	  focusCells[2] = new AtomGridCell(focusX,   focusY, focusZ-1);
	  focusCells[3] = new AtomGridCell(focusX-1, focusY, focusZ-1);
	}
	else if (z > 0) {
	  focusCells[2] = new AtomGridCell(focusX,   focusY, focusZ+1);
	  focusCells[3] = new AtomGridCell(focusX-1, focusY, focusZ+1);
	}
      }
      else if (x > 0) {
	focusCells[1] = new AtomGridCell(focusX+1, focusY, focusZ);
	if (z < 0) {
	  focusCells[2] = new AtomGridCell(focusX,   focusY, focusZ-1);
	  focusCells[3] = new AtomGridCell(focusX+1, focusY, focusZ-1);
	}
	else if (z > 0) {
	  focusCells[2] = new AtomGridCell(focusX,   focusY, focusZ+1);
	  focusCells[3] = new AtomGridCell(focusX+1, focusY, focusZ+1);
	}
      }
    }
    
    else if (z == 0) {
      if (x < 0) {
	focusCells[1] = new AtomGridCell(focusX-1, focusY, focusZ);
	if (y < 0) {
	  focusCells[2] = new AtomGridCell(focusX,   focusY-1, focusZ);
	  focusCells[3] = new AtomGridCell(focusX-1, focusY-1, focusZ);
	}
	else if (y > 0) {
	  focusCells[2] = new AtomGridCell(focusX,   focusY+1, focusZ);
	  focusCells[3] = new AtomGridCell(focusX-1, focusY+1, focusZ);
	}
      }
      else if (x > 0) {
	focusCells[1] = new AtomGridCell(focusX+1, focusY, focusZ);
	if (y < 0) {
	  focusCells[2] = new AtomGridCell(focusX,   focusY-1, focusZ);
	  focusCells[3] = new AtomGridCell(focusX+1, focusY-1, focusZ);
	}
	else if (y > 0) {
	  focusCells[2] = new AtomGridCell(focusX,   focusY+1, focusZ);
	  focusCells[3] = new AtomGridCell(focusX+1, focusY+1, focusZ);
	}
      }
    }
    return 1;

  case 3:
    if (x < 0) {
      focusCells[1] = new AtomGridCell(focusX-1, focusY, focusZ);
      if (y < 0) {
	focusCells[2] = new AtomGridCell(focusX,   focusY-1, focusZ);
	focusCells[3] = new AtomGridCell(focusX-1, focusY-1, focusZ);
	if (z < 0) {
	  focusCells[4] = new AtomGridCell(focusX,   focusY,   focusZ-1);
	  focusCells[5] = new AtomGridCell(focusX,   focusY-1, focusZ-1);
	  focusCells[6] = new AtomGridCell(focusX-1, focusY,   focusZ-1);
	  focusCells[7] = new AtomGridCell(focusX-1, focusY-1, focusZ-1);
	}
	else if (z > 0) {
	  focusCells[4] = new AtomGridCell(focusX,   focusY,   focusZ+1);
	  focusCells[5] = new AtomGridCell(focusX,   focusY-1, focusZ+1);
	  focusCells[6] = new AtomGridCell(focusX-1, focusY,   focusZ+1);
	  focusCells[7] = new AtomGridCell(focusX-1, focusY-1, focusZ+1);
	}
      }
      else if (y > 0) {
	focusCells[2] = new AtomGridCell(focusX,   focusY+1, focusZ);
	focusCells[3] = new AtomGridCell(focusX-1, focusY+1, focusZ);
	if (z < 0) {
	  focusCells[4] = new AtomGridCell(focusX,   focusY,   focusZ-1);
	  focusCells[5] = new AtomGridCell(focusX,   focusY+1, focusZ-1);
	  focusCells[6] = new AtomGridCell(focusX-1, focusY,   focusZ-1);
	  focusCells[7] = new AtomGridCell(focusX-1, focusY+1, focusZ-1);
	}
	else if (z > 0) {
	  focusCells[4] = new AtomGridCell(focusX,   focusY,   focusZ+1);
	  focusCells[5] = new AtomGridCell(focusX,   focusY+1, focusZ+1);
	  focusCells[6] = new AtomGridCell(focusX-1, focusY,   focusZ+1);
	  focusCells[7] = new AtomGridCell(focusX-1, focusY+1, focusZ+1);
	}
      }
    }

    else if (x > 0) {
      focusCells[1] = new AtomGridCell(focusX+1, focusY, focusZ);
      if (y < 0) {
	focusCells[2] = new AtomGridCell(focusX,   focusY-1, focusZ);
	focusCells[3] = new AtomGridCell(focusX+1, focusY-1, focusZ);
	if (z < 0) {
	  focusCells[4] = new AtomGridCell(focusX,   focusY,   focusZ-1);
	  focusCells[5] = new AtomGridCell(focusX,   focusY-1, focusZ-1);
	  focusCells[6] = new AtomGridCell(focusX+1, focusY,   focusZ-1);
	  focusCells[7] = new AtomGridCell(focusX+1, focusY-1, focusZ-1);
	}
	else if (z > 0) {
	  focusCells[4] = new AtomGridCell(focusX,   focusY,   focusZ+1);
	  focusCells[5] = new AtomGridCell(focusX,   focusY-1, focusZ+1);
	  focusCells[6] = new AtomGridCell(focusX+1, focusY,   focusZ+1);
	  focusCells[7] = new AtomGridCell(focusX+1, focusY-1, focusZ+1);
	}
      }
      else if (y > 0) {
	focusCells[2] = new AtomGridCell(focusX,   focusY+1, focusZ);
	focusCells[3] = new AtomGridCell(focusX+1, focusY+1, focusZ);
	if (z < 0) {
	  focusCells[4] = new AtomGridCell(focusX,   focusY,   focusZ-1);
	  focusCells[5] = new AtomGridCell(focusX,   focusY+1, focusZ-1);
	  focusCells[6] = new AtomGridCell(focusX+1, focusY,   focusZ-1);
	  focusCells[7] = new AtomGridCell(focusX+1, focusY+1, focusZ-1);
	}
	else if (z > 0) {
	  focusCells[4] = new AtomGridCell(focusX,   focusY,   focusZ+1);
	  focusCells[5] = new AtomGridCell(focusX,   focusY+1, focusZ+1);
	  focusCells[6] = new AtomGridCell(focusX+1, focusY,   focusZ+1);
	  focusCells[7] = new AtomGridCell(focusX+1, focusY+1, focusZ+1);
	}
      }
    }
    return 1;
  }
  
  return 0;
}

