
#ifndef BASEGRAPH_H
#define BASEGRAPH_H

#include "baseVertexNode.h"
#include "baseVertexNodeTree.h"
#include "connectionMatrix.h"
#include "coordinate3D.h"
#include "genericList.h"
//#include "genericList.c"
#include "groupId.c"
//#include "triangleEdgeTree.h"
#include "triangleGraph.h"
//#include "tetrahedronGraph.h"

#include <stdio.h>


class BaseGraph {

 public:
  /** Constructor. */
  BaseGraph(int mVC);

  /** Destructor. */
  ~BaseGraph();


  /**
   * Get vertex count.
   * @return int: number of vertices in the graph.
   */
  int getVertexCount();

  /**
   * Get edge count.
   * @return int: number of edges in the graph.
   */
  int getEdgeCount();

  /**
   * Get ordered Coordinate3D data for the vertices.
   * @return Coordinate3D**: array of Coordinate3D* corresponding to the vertices.
   */
  Coordinate3D** getCoords();

  /**
   * Get ConnectionMatrix for the graph.
   * @return ConnectionMatrix*: information about the connectivity (edges) in the graph.
   */
  ConnectionMatrix* getConnectionMatrix();

  /**
   * Add nodes to the graph.
   * @param coord Coordinate3D*: coordinate for new node.
   * @return int: 0 on fail, 1 on success.
   */
  int addVertex(Coordinate3D* coord);

  /**
   * Add edges to the graph.
   * @param coord1 Coordinate3D*: first coordinate for new edge.
   * @param coord2 Coordinate3D*: second coordinate for new edge.
   * @return int: 0 on fail, 1 on success.
   */
  int addEdge(Coordinate3D* coord1, Coordinate3D* coord2);

  /**
   * Activate all nodes in the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int activateAll();

  /**
   * Deactivate all nodes in the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivateAll();

  /**
   * Deactivate outlier edges.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivateLongEdges();

  /**
   * Update or initially set the node IDs.
   * @return int: 0 on fail, 1 on success.
   */
  int refreshIds();

  /**
   * Update or initially set the nodes' group IDs.
   * @return int: 0 if not active, 1 if active.
   */
  int refreshGroupIds();

  /**
   * Create and return a TriangleGraph based on this BaseGraph.
   * @return TriangleGraph*: graph where a triangle is a node
   */
  TriangleGraph* getTriangleGraph();
  
  /**
   *
   * @return int: 0 if not active, 1 if active.
   */
  int getSharedNeighbors(BaseVertexNode* node1,
			 BaseVertexNode* node2,
			 GenericList<BaseVertexNode**>* triangleList);

  /**
   * Get the difference in distance between corresponding vertices from two graphs.  This should be used when the graphs have the same number of vertices, and they are tessellations of the same system in different states.
   * @param graph BaseGraph*: graph to which the comparison is made.
   * @return int: 0 on fail, 1 on success.
   */
  int compareVertexPositions(BaseGraph* graph);

  /**
   * Read in a tcl tessellation file to build a corresponding graph.
   * @param infile FILE*: tcl tessellation file from which to read the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int readTessellationScript(FILE* infile);

  /**
   * Print a tcl tessellation file for the active vertices and edges of this graph.
   * @param outfile FILE*: tcl tessellation file in which to write the graph.
   * @param colorByValue int: whether to color the tessellation by value or not.
   * @return int: 0 on fail, 1 on success.
   */
  int printTessellationScript(FILE* outfile, int colorByValue);

  /**
   * Print a tcl tessellation file for the active vertices and edges of this graph.
   * @param outfile FILE*: tcl tessellation file in which to write the graph.
   * @param colorByValue int: whether to color the tessellation by value or not.
   * @return int: 0 on fail, 1 on success.
   */
  int printTessellationScript2(FILE* outfile, int colorByValue);

  /**
   * Print a tcl tessellation file for the active vertices and edges of this graph.
   * @param outfile FILE*: tcl tessellation file in which to write the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int printTessellationScript(FILE* outfile);


  /**
   * DEBUG PROC. Print out of node state.
   */
  int print();
  
 private:
  BaseVertexNode** nodes;  // Array of nodes
  BaseVertexNodeTree* nodeTree;  // Lookup: Coordinate3D to BaseVertexNode
  GenericList<BaseVertexNode*>* groups;  // Connected subgraphs
  int vertexCount;
  int edgeCount;
  int activeVertexCount;
  int activeEdgeCount;
  int groupCount;
  int activeGroupCount;
  int maxVertexCount;
  //BaseVertexGraph** vertices;
  //BaseEdgeGraph**   edges;
  //TriangleGraph* triangles;       // For destruction purposes
  //TetrahedronGraph* tetrahedrons; // For destruction purposes

};

#endif
