
#include "tetrahedronEdgeTree.h"


/** Constructor. */
TetrahedronEdgeTree::TetrahedronEdgeTree(TetrahedronEdge* e) :
  edge(e), child1(0), child2(0) {

  edgeList = new GenericList<TetrahedronGraphNode*>();

  return;
}


TetrahedronEdgeTree::TetrahedronEdgeTree(TetrahedronGraphNode* n) {

  Coordinate3D* coord1 = n->getCoordinate3D(0);
  Coordinate3D* coord2 = n->getCoordinate3D(1);
  Coordinate3D* coord3 = n->getCoordinate3D(2);
  Coordinate3D* coord4 = n->getCoordinate3D(3);


  if ( coord1->isGreaterThan(coord2) == 1 ||
       coord2->isGreaterThan(coord3) == 1 ||
       coord3->isGreaterThan(coord4) == 1 ) {
    printf("Error: TetrahedronEdgeTree::TetrahedronEdgeTree - coordinates out of order\n");
    return;
  }

  // Set edge to the second edge of the tetrahedron
  // XXX - COULD IMPROVE BY RANDOMLY PICKING 2ND OR 3RD EDGE TO SPLIT ON (see ***)
  edge = new TetrahedronEdge(coord1,coord2,coord4);
  TetrahedronEdge* tempEdge1 = new TetrahedronEdge(coord1,coord2,coord3);
  TetrahedronEdge* tempEdge2 = new TetrahedronEdge(coord1,coord3,coord4);
  TetrahedronEdge* tempEdge3 = new TetrahedronEdge(coord2,coord3,coord4);
  
  edgeList = new GenericList<TetrahedronGraphNode*>();
  edgeList->push(n);

  child1 = new TetrahedronEdgeTree(tempEdge1);
  child1->insert(tempEdge1,n);

  child2 = new TetrahedronEdgeTree(tempEdge2);
  child2->insert(tempEdge2,n);
  // XXX - ***
  child2->insert(tempEdge3,n);

  return;
}


/** Destructor. */
TetrahedronEdgeTree::~TetrahedronEdgeTree() {

  delete child1;
  delete child2;
  delete edge;
  delete edgeList;

  return;
}


/**
 * Add a node to the tree. Look at the 4 faces of the tetrahedron.  For each face, store a link to the tetrahedron at the corresponding node in the tree.
 * @param node TetrahedronGraphNode*: node to add.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronEdgeTree::insert(TetrahedronGraphNode* node) {

  Coordinate3D* coord1 = node->getCoordinate3D(0);
  Coordinate3D* coord2 = node->getCoordinate3D(1);
  Coordinate3D* coord3 = node->getCoordinate3D(2);
  Coordinate3D* coord4 = node->getCoordinate3D(3);

  if ( coord1->isGreaterThan(coord2) == 1 ||
       coord2->isGreaterThan(coord3) == 1 ||
       coord3->isGreaterThan(coord4) == 1 ) {
    printf("Error: TetrahedronEdgeTree::TetrahedronEdgeTree - coordinates out of order\n");
    return 0;
  }

  // Send edges off with node
  TetrahedronEdge* tempEdge1 = new TetrahedronEdge(coord1,coord2,coord3);
  TetrahedronEdge* tempEdge2 = new TetrahedronEdge(coord1,coord2,coord4);
  TetrahedronEdge* tempEdge3 = new TetrahedronEdge(coord1,coord3,coord4);
  TetrahedronEdge* tempEdge4 = new TetrahedronEdge(coord2,coord3,coord4);

  int result = 0;
  result += insert(tempEdge1, node);
  result += insert(tempEdge2, node);
  result += insert(tempEdge3, node);
  result += insert(tempEdge4, node);

  if (result < 4) {
    return 0;
  }

  return 1;
}


/**
 * Add a node to the tree. Look at the 4 faces of the tetrahedron.  For each face, store a link to the tetrahedron at the corresponding node in the tree.
 * @param edge TetrahedronEdge*: face of tetrahedron.
 * @param triangle TetrahedronGraphNode*: tetrahedron to add.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronEdgeTree::insert(TetrahedronEdge* e, TetrahedronGraphNode* tetrahedron) {

  if (edge->equals(e) == 1) {
    edgeList->push(tetrahedron);
  }
  if (edge->isGreaterThan(e) == 1) {
    if (child1 == 0) {
      child1 = new TetrahedronEdgeTree(e);
    }
    return child1->insert(e, tetrahedron);
  }
  if (edge->isLessThan(e) == 1) {
    if (child2 == 0) {
      child2 = new TetrahedronEdgeTree(e);
    }
    return child2->insert(e, tetrahedron);
  }

  return 0;
}


/**
 * Connect the nodes that correspond to tetrahedra sharing a face.
 * @return GenericList<IdPair*>*: list of TetrahedronNode id pairs for edge creation.
 */
int TetrahedronEdgeTree::createEdges(GenericList<IdPair*>* idList) {

  TetrahedronGraphNode* tempNode1 = 0;
  TetrahedronGraphNode* tempNode2 = 0;
  int id1 = 0;
  int id2 = 0;

  // tempEdgeList: store edgeList elements for later retrieval.
  GenericList<TetrahedronGraphNode*>* tempEdgeList = edgeList->clone();

  // Create IdPairs for each triangle pair sharing the current edge.
  while (tempEdgeList->getLength() > 0) {
    tempNode1 = tempEdgeList->front();
    id1 = tempNode1->getId();
    tempNode2 = tempEdgeList->getFirst();
    while (tempNode2 != 0) {
      id2 = tempNode2->getId();
      IdPair* idPair = new IdPair();
      idPair->x = id1;
      idPair->y = id2;
      idList->push(idPair);
      tempNode2 = tempEdgeList->getNext();
    }
  }

  delete tempEdgeList;
  
  int result = 0;
  if (child1 != 0) {
    result = child1->createEdges(idList);
    if (result == 0) {
      printf("Error: TetrahedronEdgeTree::createEdges: child1 Missing the creation of edges\n");
    }
  }
  if (child2 != 0) {
    result = child2->createEdges(idList);
    if (result == 0) {
      printf("Error: TetrahedronEdgeTree::createEdges: child2 Missing the creation of edges\n");
    }
  }

  return 1;
}
