
#include "tetrahedronGraph.h"


/** Constructor. */
TetrahedronGraph::TetrahedronGraph(int mVC) :
  vertexCount(0), edgeCount(0),
  activeVertexCount(0), activeEdgeCount(0), groupCount(0),
  activeGroupCount(0), maxVertexCount(mVC) {
  
  nodes = new TetrahedronGraphNode*[maxVertexCount];
  int i = 0;
  for (; i<maxVertexCount; i++) {
    nodes[i] = 0;
  }

  groups = new GenericList<TetrahedronGraphNode*>();

  return;
}


/** Destructor. */
TetrahedronGraph::~TetrahedronGraph() {

  int i = 0;
  for (; i<vertexCount; i++) {
    delete nodes[i];
  }

  delete nodes;
  delete groups;

  return;
}


/**
 * Add nodes to the graph.
 * @param node1 TriangleGraphNode*: first face of tetrahedron.
 * @param node2 TriangleGraphNode*: second face of tetrahedron.
 * @param node3 TriangleGraphNode*: third face of tetrahedron.
 * @param node4 TriangleGraphNode*: fourth face of tetrahedron.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::addVertex(TriangleGraphNode* node1, TriangleGraphNode* node2,
				TriangleGraphNode* node3, TriangleGraphNode* node4) {


  //printf("->TetrahedronGraph::addVertex\n");

  if (vertexCount >= maxVertexCount) {
    //printf("<-TetrahedronGraph::addVertex: 0\n");
    return 0;
  }

  //printf("  hey1\n");
  if (node1 == 0 || node2 == 0 || node3 == 0 || node4 == 0) {
    printf("Error: TetrahedronGraph::addVertex: Have an unset node.\n");
    //printf("<-TetrahedronGraph::addVertex: 0\n");
    return 0;
  }

  //printf("  hey2\n");
  TetrahedronGraphNode* tempNode = new TetrahedronGraphNode(node1,node2,node3,node4);
  tempNode->setId(vertexCount);
  nodes[vertexCount] = tempNode;
  
  //printf("  hey3\n");
  // Check that nodes are connected by active edges; if not, deactivate the tetrahedron.
  if (!node1->isActive(node2) ||
      !node1->isActive(node3) ||
      !node1->isActive(node4) ||
      !node2->isActive(node3) ||
      !node2->isActive(node4) ||
      !node3->isActive(node4)) {
    tempNode->deactivate();
  }

  vertexCount++;
  activeVertexCount++;

  printf("<-TetrahedronGraph::addVertex %d\n",vertexCount);

  return 1;
}


/**
 * Find and add edges to the graph by searching for tetrahedra that share a face.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::addEdges() {

  if (vertexCount == 0) {
    return 0;
  }
    
  // Set up the TetrahedronEdgeTree for edge determination.
  TetrahedronEdgeTree* tetraEdgeTree = new TetrahedronEdgeTree(nodes[0]);
  
  int i=1;
  for (; i<vertexCount; i++) {
    tetraEdgeTree->insert(nodes[i]);
  }
  
  GenericList<IdPair*>* idList = new GenericList<IdPair*>();
  int result = tetraEdgeTree->createEdges(idList);
  delete tetraEdgeTree;
  if (result == 0) {
    printf("Error: TetrahedronGraph::addEdges\n");
  }
  
  // Create edges between TetrahedronGraphNodes.
  while (idList->getLength() > 0) {
    IdPair* nodeId = idList->front();
    addEdge(nodeId->x, nodeId->y);
    delete nodeId;
  }
  
  delete idList;

  return 1;
}


/**
 * Add an edge between two nodes in the graph.
 * @param id1 int: ID of first TetrahedronGraphNode
 * @param id2 int: ID of second TetrahedronGraphNode
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::addEdge(int id1, int id2) {

  // if (ids are equal) {don't add edge from vertex to itself}
  if (id1 == id2) {
    return 0;
  }

  // if (ids are out of bounds) {return 0;}
  if (id1 < 0 || id1 > vertexCount ||
      id2 < 0 || id2 > vertexCount) {
    return 0;
  }

  TetrahedronGraphNode* tempNode1 = nodes[id1];
  TetrahedronGraphNode* tempNode2 = nodes[id2];

  printf("  Adding edge: (%d,%d)\n",id1,id2);

  if (tempNode1 != 0 && tempNode2 != 0) {
    int result = tempNode1->addVertex(tempNode2);
    if (result != 0) {
      result = tempNode2->addVertex(tempNode1);
    }
    if (result != 0) {
      edgeCount++;
      if (tempNode1->isActive() &&
	  tempNode2->isActive()) {
	activeEdgeCount++;
      }
    }
    return result;
  }

  return 0;
}


/**
 * Get volume.
 * @return float: sum of volumes of tetrahedra in the graph.
 */
float TetrahedronGraph::getVolume() {

  float volume = 0.0;
  float tempVolume = 0.0;
  float tempVolume2 = 0.0;

  int i=0;
  for (; i<vertexCount; i++) {
    tempVolume = nodes[i]->getVolume();
    tempVolume2 = nodes[i]->getVolume2();
    volume += tempVolume;
    printf("   tempVolume = %3.3f, tempVolume2 = %3.3f, volume = %3.3f\n",tempVolume,tempVolume2,volume);
  }

  return volume;
}


/**
 * Activate all nodes in the graph.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::activateAll() {

  int i = 0;
  for (; i<vertexCount; i++) {
    nodes[i]->activate();
  }

  return 1;
}


/**
 * Deactivate all nodes in the graph.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::deactivateAll() {

  int i = 0;
  for (; i<vertexCount; i++) {
    nodes[i]->deactivate();
  }

  return 1;
}


/**
 * Set "type" attributes for all the triangles.  Triangles shared by two tetrahedra are INTERNAL, those part of only one tetrahedron are EXTERNAL, and free triangles are DOUBLE.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::setTriangleTypes() {
  
  printf("->TetrahedronGraph::setTriangleTypes\n");

  // Loop through tetrahedra setting the types of their faces.
  // Set EXTERNAL triangles.
  int i = 0;
  int j = 0;
  for (i=0; i<vertexCount; i++) {
    printf("  node[%d]:",i);
    for (j=0; j<4; j++) {
      printf(" %d",nodes[i]->getFace(j)->getId());
      nodes[i]->getFace(j)->setType(TriangleGraphNode::EXTERNAL);
    }
    printf("\n");
  }

  // Set INTERNAL triangles.
  int result = 0;
  for (i=0; i<vertexCount; i++) {
    result += nodes[i]->setInternalTriangleTypes();
  }

  if (result > 0) {
    result = 1;
  }
  
  printf("<-TetrahedronGraph::setTriangleTypes\n");

  return result;
}


/**
 * Deactivate TriangleGraphNodes within the interior of solid polyhedra.
 * @return int: 0 on fail, 1 on success.
 */
/*
  int TetrahedronGraph::deactivateInterior() {
  
  return 0;
  }
*/


/**
 * Update or initially set the node IDs.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::refreshIds() {

  return 0;
}


/**
 * Update or initially set the nodes' group IDs.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::refreshGroupIds() {

  return 0;
}


/**
 * Print a tcl tessellation file for the active vertices and edges of this graph.
 * @param outfile FILE*: tcl tessellation file in which to write the graph.
 * @param colorByValue int: whether to color the tessellation by value or not.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::printTessellationScript(FILE* outfile, int colorByValue) {




  return 1;
}


/**
 * Print a tcl tessellation file for the active vertices and edges of this graph.
 * @param outfile FILE*: tcl tessellation file in which to write the graph.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraph::printTessellationScript(FILE* outfile) {

  return printTessellationScript(outfile, 0);
}


/**
 * DEBUG PROC. Print out of node state.
 */
int TetrahedronGraph::print() {



  return 1;
}
