
#ifndef TETRAHEDRONGRAPH_H
#define TETRAHEDRONGRAPH_H

#include "genericList.h"
#include "groupId.c"
#include "tetrahedronEdgeTree.h"
#include "tetrahedronGraphNode.h"
#include "triangleGraphNode.h"

#include <stdio.h>


class TetrahedronGraph {

 public:
  /** Constructor. */
  TetrahedronGraph(int mVC);

  /** Destructor. */
  ~TetrahedronGraph();

  /**
   * Add nodes to the graph.
   * @param node1 TriangleGraphNode*: first face of tetrahedron.
   * @param node2 TriangleGraphNode*: second face of tetrahedron.
   * @param node3 TriangleGraphNode*: third face of tetrahedron.
   * @param node4 TriangleGraphNode*: fourth face of tetrahedron.
   * @return int: 0 on fail, 1 on success.
   */
  int addVertex(TriangleGraphNode* node1, TriangleGraphNode* node2,
		TriangleGraphNode* node3, TriangleGraphNode* node4);

  /**
   * Find and add edges to the graph by searching for tetrahedra that share a face.
   * @return int: 0 on fail, 1 on success.
   */
  int addEdges();

  /**
   * Add an edge between two nodes in the graph.
   * @param id1 int: ID of first TetrahedronGraphNode
   * @param id2 int: ID of second TetrahedronGraphNode
   * @return int: 0 on fail, 1 on success.
   */
  int addEdge(int id1, int id2);

  /**
   * Get volume.
   * @return float: sum of volumes of tetrahedra in the graph.
   */
  float getVolume();

  /**
   * Activate all nodes in the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int activateAll();

  /**
   * Deactivate all nodes in the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivateAll();

  /**
   * Set "type" attributes for all the triangles.  Triangles shared by two tetrahedra are INTERIOR, those part of only one tetrahedron are EXTERIOR, and free triangles are DOUBLE.
   * @return int: 0 on fail, 1 on success.
   */
  int setTriangleTypes();

  /**
   * Deactivate TriangleGraphNodes within the interior of solid polyhedra.
   * @return int: 0 on fail, 1 on success.
   */
  //int deactivateInterior();

  /**
   * Update or initially set the node IDs.
   * @return int: 0 on fail, 1 on success.
   */
  int refreshIds();

  /**
   * Update or initially set the nodes' group IDs.
   * @return int: 0 on fail, 1 on success.
   */
  int refreshGroupIds();

  /**
   * Print a tcl tessellation file for the active vertices and edges of this graph.
   * @param outfile FILE*: tcl tessellation file in which to write the graph.
   * @param colorByValue int: whether to color the tessellation by value or not.
   * @return int: 0 on fail, 1 on success.
   */
  int printTessellationScript(FILE* outfile, int colorByValue);

  /**
   * Print a tcl tessellation file for the active vertices and edges of this graph.
   * @param outfile FILE*: tcl tessellation file in which to write the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int printTessellationScript(FILE* outfile);


  /**
   * DEBUG PROC. Print out of node state.
   */
  int print();

 private:
  TetrahedronGraphNode** nodes;  // Array of nodes
  GenericList<TetrahedronGraphNode*>* groups;  // Connected subgraphs
  int vertexCount;
  int edgeCount;
  int activeVertexCount;
  int activeEdgeCount;
  int groupCount;
  int activeGroupCount;
  int maxVertexCount;
};

#endif
