
#ifndef TETRAHEDRONGRAPHNODE_H
#define TETRAHEDRONGRAPHNODE_H

#include "coordinate3D.h"
#include "genericList.h"
#include "groupId.c"
#include "triangleGraphNode.h"

struct TetrahedronNode {
  // Nodes in order from least to greatest.
  TriangleGraphNode* node1;
  TriangleGraphNode* node2;
  TriangleGraphNode* node3;
  TriangleGraphNode* node4;

  // Coordinate points in order from least to greatest.
  Coordinate3D* vertex1;
  Coordinate3D* vertex2;
  Coordinate3D* vertex3;
  Coordinate3D* vertex4;

  TetrahedronNode(TriangleGraphNode* w,
		  TriangleGraphNode* x,
		  TriangleGraphNode* y,
		  TriangleGraphNode* z) {

    //printf("->TetrahedronNode::Constructor\n");

    // Set up TriangleGraphNodes.
    GenericList<TriangleGraphNode*>* tempList1 = new GenericList<TriangleGraphNode*>();
    GenericList<TriangleGraphNode*>* tempList2 = new GenericList<TriangleGraphNode*>();
    GenericList<TriangleGraphNode*>* tempList3 = new GenericList<TriangleGraphNode*>();
    
    if (w->isLessThan(x)) {
      tempList1->push(w);
      tempList1->push(x);
    }
    else {
      tempList1->push(x);
      tempList1->push(w);
    }
    
    if (y->isLessThan(z)) {
      tempList1->push(y);
      tempList1->push(z);
    }
    else {
      tempList1->push(z);
      tempList1->push(y);
    }
    
    int i=0;
    for (; i<4; i++) {
      if (tempList1->getLength() == 0) {
	tempList3->push(tempList2->front());
      }
      else if (tempList2->getLength() == 0) {
	tempList3->push(tempList1->front());
      }
      else if (tempList1->getFirst()->isLessThan(tempList2->getFirst())) {
	tempList3->push(tempList1->front());
      }
      else {
	tempList3->push(tempList2->front());
      }
    }

    node1 = tempList3->front();
    node2 = tempList3->front();
    node3 = tempList3->front();
    node4 = tempList3->front();

    delete tempList1;
    delete tempList2;
    delete tempList3;

    // Set up Coordinate3Ds.
    // Determine 4 unique vertices.
    Coordinate3D* a = node1->getCoordinate3D(0);
    Coordinate3D* b = node1->getCoordinate3D(1);
    Coordinate3D* c = node1->getCoordinate3D(2);

    Coordinate3D* d = 0;
    for (i=0; i<3; i++) {
      if (node1->hasCoordinate3D(node2->getCoordinate3D(i)) == 0) {
	d = node2->getCoordinate3D(i);
	break;
      }
    }

    GenericList<Coordinate3D*>* tempVList1 = new GenericList<Coordinate3D*>();
    GenericList<Coordinate3D*>* tempVList2 = new GenericList<Coordinate3D*>();
    GenericList<Coordinate3D*>* tempVList3 = new GenericList<Coordinate3D*>();
    
    if (a->isLessThan(b)) {
      tempVList1->push(a);
      tempVList1->push(b);
    }
    else {
      tempVList1->push(b);
      tempVList1->push(a);
    }
    
    if (c->isLessThan(d)) {
      tempVList1->push(c);
      tempVList1->push(d);
    }
    else {
      tempVList1->push(d);
      tempVList1->push(c);
    }
    
    for (i=0; i<4; i++) {
      if (tempVList1->getLength() == 0) {
	tempVList3->push(tempVList2->front());
      }
      else if (tempVList2->getLength() == 0) {
	tempVList3->push(tempVList1->front());
      }
      else if (tempVList1->getFirst()->isLessThan(tempVList2->getFirst())) {
	tempVList3->push(tempVList1->front());
      }
      else {
	tempVList3->push(tempVList2->front());
      }
    }

    vertex1 = tempVList3->front();
    vertex2 = tempVList3->front();
    vertex3 = tempVList3->front();
    vertex4 = tempVList3->front();

    delete tempVList1;
    delete tempVList2;
    delete tempVList3;

    //printf("<-TetrahedronNode::Constructor\n");
    return;
  }

};


class TetrahedronGraphNode {

  /**
   * Edges for TetrahedronGraphNode.
   */
  struct TetrahedronGraphEdge {
    TetrahedronGraphNode* vertex;
    int active;
    
    TetrahedronGraphEdge(TetrahedronGraphNode* v) :
      vertex(v), active(1) {}
  };
  

 public:
  /** Constructor. */
  TetrahedronGraphNode(TriangleGraphNode* node1,
		       TriangleGraphNode* node2,
		       TriangleGraphNode* node3,
		       TriangleGraphNode* node4);
  
  /** Destructor. */
  ~TetrahedronGraphNode();

  /**
   * Get first node.
   * @return TetrahedronGraphNode*: first node, 0 on fail.
   */
  TetrahedronGraphNode* getFirstVertex();

  /**
   * Get next node.
   * @return TetrahedronGraphNode*: next node, 0 on fail.
   */
  TetrahedronGraphNode* getNextVertex();

  /**
   * Get the 3D coordinates.
   * @param i int: coordinate 0, 1, 2, or 3.
   * @return Coordinate3D*: tetrahedron corner.
   */
  Coordinate3D* getCoordinate3D(int i);

  /**
   * Get the 3D coordinates.
   * @param i int: face 0, 1, 2, or 3.
   * @param j int: index 0, 1, or 2.
   * @return Coordinate3D*: triangle corner.
   */
  Coordinate3D* getCoordinate3D(int i, int j);

  /**
   * Get the triangular tetrahedron face.
   * @param i int: face 0, 1, 2, or 3.
   * @return TriangleGraphNode*: face.
   */
  TriangleGraphNode* getFace(int i);

  /**
   * Get the value of the node.
   * @param i int: face 0, 1, or 2, or 3.
   * @param j int: index 0, 1, or 2.
   * @return float: value.
   */
  float getValue(int i, int j);

  /**
   * Get ID.
   * @return int: ID.
   */
  int getId();

  /**
   * Set ID.
   * @param i int: ID.
   * @return int: 0 on fail, 1 on success.
   */
  int setId(int i);

  /**
   * Get group ID.
   * @return GroupId*: group ID.
   */
  GroupId* getGroupId();

  /**
   * Set group ID.
   * @param gId GroupId*: group ID.
   * @return int: 0 on fail, 1 on success.
   */
  int setGroupId(GroupId* gId);

  /**
   * Get vertex count.
   * @return int: vertex count.
   */
  int getVertexCount();

  /**
   * Get active vertex count.
   * @return int: active vertex count.
   */
  int getActiveVertexCount();

  /**
   * Set "type" attributes of internal triangles to INTERNAL.
   * @return int: 0 on fail, 1 on success.
   */
  int setInternalTriangleTypes();

  /**
   * Get face shared by two neighboring tetrahedra.
   * @param tetraNode TetrahedronGraphNode*: node this tetrahedron connects to.
   * @return TriangleGraphNode*: face that the two tetrahedra share.
   */
  TriangleGraphNode* getSharedFace(TetrahedronGraphNode* tetraNode);

  /**
   * Get volume.
   * @return float: volume of this tetrahedron.
   */
  float getVolume();

  /**
   * Get volume.
   * @return float: volume of this tetrahedron.
   */
  float getVolume2();

  /**
   * Activate this node.
   * @return int: 0 on fail, 1 on success.
   */
  int activate();

  /**
   * Deactivate this node.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivate();

  /**
   * Activate the edge leading to the specified node.
   * @param b TetrahedronGraphNode*: node the edge connects to.
   * @return int: 0 on fail, 1 on success.
   */
  int activateEdge(TetrahedronGraphNode* b);

  /**
   * Deactivate the edge leading to the specified node.
   * @param b TetrahedronGraphNode*: node the edge connects to.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivateEdge(TetrahedronGraphNode* b);

  /**
   * Check whether this node is active or not.
   * @return int: 0 if not active, 1 if active.
   */
  int isActive();

  /**
   * Check whether the edge leading to the specified node is active.
   * @param b TetrahedronGraphNode*: node the edge connects to.
   * @return int: 0 if not active, 1 if active.
   */
  int isActive(TetrahedronGraphNode* b);

  /**
   * DEBUG PROC. Printout of node state.
   */
  int print();


  //protected:
  /**
   * Add a vertex with an edge to this node.
   * @param b TetrahedronGraphNode*: node to add.
   * @return int: 0 on fail, 1 on success.
   */
  int addVertex(TetrahedronGraphNode* b);
  
  /**
   * Mark this node.
   * @return int: 0 on fail, 1 on success.
   */
  int mark();

  /**
   * Unmark this node.
   * @return int: 0 on fail, 1 on success.
   */
  int unmark();

  /**
   * Check whether this node is marked or not.
   * @return int: 0 if not active, 1 if active.
   */
  int isMarked();


 private:
  TetrahedronNode* tetrahedron;
  GenericList<TetrahedronGraphNode::TetrahedronGraphEdge*>* vertices;
  GroupId* group;
  int id;
  int active;
  int marked;
  int vertexCount;
  int activeVertexCount;
  //Coordinate*[4] coords;  // Vertices
  //TriangleNode*[4] triangles;  // Faces

};

#endif
