
#include "triangleGraph.h"


/** Constructor. */
TriangleGraph::TriangleGraph(int mVC) :
  vertexCount(0), edgeCount(0),
  activeVertexCount(0), activeEdgeCount(0), groupCount(0),
  activeGroupCount(0), maxVertexCount(mVC) {
  
  nodes = new TriangleGraphNode*[maxVertexCount];
  int i = 0;
  for (; i<maxVertexCount; i++) {
    nodes[i] = 0;
  }

  groups = new GenericList<TriangleGraphNode*>();

  return;
}


/** Destructor. */
TriangleGraph::~TriangleGraph() {

  int i = 0;
  for (; i<vertexCount; i++) {
    delete nodes[i];
  }

  /*
    if (nodeTree != 0) {
    delete nodeTree;
    }
  */

  delete nodes;
  delete groups;

  return;
}


/**
 * Add nodes to the graph.
 * @param node1 BaseVertexNode*: first corner of triangle.
 * @param node2 BaseVertexNode*: second corner of triangle.
 * @param node3 BaseVertexNode*: third corner of triangle.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::addVertex(BaseVertexNode* node1,
			     BaseVertexNode* node2,
			     BaseVertexNode* node3) {

  //printf("->TriangleGraph::addVertex\n");

  if (vertexCount >= maxVertexCount) {
    //printf("<-TriangleGraph::addVertex: 0\n");
    return 0;
  }

  if (node1 == 0 || node2 == 0 || node3 == 0) {
    printf("Error: TriangleGraph::addVertex: Have an unset node.\n");
    //printf("<-TriangleGraph::addVertex: 0\n");
    return 0;
  }

  TriangleGraphNode* tempNode = new TriangleGraphNode(node1,node2,node3);
  tempNode->setId(vertexCount);
  nodes[vertexCount] = tempNode;
  
  if (!node1->isActive(node2) ||
      !node1->isActive(node3) ||
      !node2->isActive(node3)) {
    tempNode->deactivate();
  }

  /*
    if (nodeTree != 0) {
    nodeTree->insert(nodes[vertexCount]);
    }
    else {
    nodeTree = new BaseVertexNodeTree(nodes[vertexCount]);
    }
  */

  vertexCount++;
  activeVertexCount++;

  //printf("<-TriangleGraph::addVertex\n");

  return 1;
}


/**
 * Find and add edges to the graph by searching for triangles that share a side.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::addEdges() {

  if (vertexCount == 0) {
    return 0;
  }

  // Set up the TriangleEdgeTree for edge determination.
  TriangleEdgeTree* triEdgeTree = new TriangleEdgeTree(nodes[0]);

  int i=1;
  for (; i<vertexCount; i++) {
    triEdgeTree->insert(nodes[i]);
  }

  GenericList<IntPair*>* idList = new GenericList<IntPair*>();
  int result = triEdgeTree->createEdges(idList);
  delete triEdgeTree;
  if (result == 0) {
      printf("Error: TriangleGraph::addEdges\n");
  }
  
  // Create edges between TriangleGraphNodes.
  while (idList->getLength() > 0) {
    IntPair* nodeId = idList->front();
    addEdge(nodeId->x, nodeId->y);
    delete nodeId;
  }

  delete idList;

  return 1;
}


/**
 * Add an edge between two nodes in the graph.
 * @param id1 int: ID of first TriangleGraphNode
 * @param id2 int: ID of second TriangleGraphNode
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::addEdge(int id1, int id2) {

  // if (ids are equal) {don't add edge from vertex to itself}
  if (id1 == id2) {
    return 0;
  }

  // if (ids are out of bounds) {return 0;}
  if (id1 < 0 || id1 > vertexCount ||
      id2 < 0 || id2 > vertexCount) {
    return 0;
  }

  TriangleGraphNode* tempNode1 = nodes[id1];
  TriangleGraphNode* tempNode2 = nodes[id2];

  //printf("  Adding edge: (%d,%d)\n",id1,id2);

  if (tempNode1 != 0 && tempNode2 != 0) {
    int result = tempNode1->addVertex(tempNode2);
    if (result != 0) {
      result = tempNode2->addVertex(tempNode1);
    }
    if (result != 0) {
      edgeCount++;
      if (tempNode1->isActive() &&
	  tempNode2->isActive()) {
	activeEdgeCount++;
      }
    }
    return result;
  }

  return 0;
}


/**
 * Get the surface area of the object.  INTERNAL triangles have no surface area, EXTERNAL triangles contribute one side, and DOUBLE triangles contribute both sides.
 * @return float: surface area.
 */
float TriangleGraph::getSurfaceArea() {

  float surfaceArea = 0.0;
  int i = 0;
  for (i=0; i<vertexCount; i++) {
    surfaceArea += nodes[i]->getSurfaceArea();
    printf("   surfaceArea: %3.3f, total: %3.3f\n",nodes[i]->getSurfaceArea(),surfaceArea);
  }

  return surfaceArea;
}


/**
 * Activate all nodes in the graph.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::activateAll() {

  int i = 0;
  for (; i<vertexCount; i++) {
    nodes[i]->activate();
  }

  return 1;
}


/**
 * Deactivate all nodes in the graph.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::deactivateAll() {

  int i = 0;
  for (; i<vertexCount; i++) {
    nodes[i]->deactivate();
  }

  return 1;
}


/**
 * Deactivate all INTERNAL type nodes in the graph.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::deactivateInternal() {

  //printf("->TriangleGraph::deactivateInternal\n");

  int i = 0;
  for (i=0; i<vertexCount; i++) {
    //printf("  type: %d, id: %d\n",nodes[i]->getType(),nodes[i]->getId());
    if (nodes[i]->getType() == TriangleGraphNode::INTERNAL) {
      nodes[i]->deactivate();
    }
  }

  //printf("<-TriangleGraph::deactivateInternal\n");

  return 0;
}


/**
 * Update or initially set the node IDs.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::refreshIds() {

  return 0;
}


/**
 * Update or initially set the nodes' group IDs.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::refreshGroupIds() {

  return 0;
}


/**
 * Create and return a TetrahedronGraph based on this BaseGraph.
 * @return TetrahedronGraph*: graph where a tetrahedron is a node
 */

TetrahedronGraph* TriangleGraph::getTetrahedronGraph() {
  
  TriangleGraphNode* currentNode = 0;
  TriangleGraphNode* tempNode = 0;
  TriangleGraphNode* side1 = 0;
  TriangleGraphNode* side2 = 0;
  TriangleGraphNode* side3 = 0;
  Coordinate3D* fourthCoord = 0;

  GenericList<TriangleGraphNode**>* tetraList = new GenericList<TriangleGraphNode**>();
  GenericList<TriangleGraphNode*>* sideList1 = 0;
  GenericList<TriangleGraphNode*>* sideList2 = 0;
  GenericList<TriangleGraphNode*>* sideList3 = 0;
  
  int i=0;
  int j=0;
  for (; i<vertexCount; i++) {
    currentNode = nodes[i];
    sideList1 = new GenericList<TriangleGraphNode*>();
    sideList2 = new GenericList<TriangleGraphNode*>();
    sideList3 = new GenericList<TriangleGraphNode*>();

    // Break edge list into 3 lists (one for each side of the triangle of concern).
    // printf("Break edge list into 3 lists (one for each side of the triangle of concern).\n");
    tempNode = currentNode->getFirstVertex();
    //printf("currentNode: %d,",currentNode->getId());
    while (tempNode != 0) {
      //printf(" %d",tempNode->getId());
      if (tempNode->getId() > currentNode->getId()) {
	//if (tempNode->getId() != currentNode->getId()) {
	if ( tempNode->hasCoordinate3D(currentNode->getCoordinate3D(0)) &&
	     tempNode->hasCoordinate3D(currentNode->getCoordinate3D(1)) ) {
	  sideList1->push(tempNode);
	}
	else if ( tempNode->hasCoordinate3D(currentNode->getCoordinate3D(0)) &&
		  tempNode->hasCoordinate3D(currentNode->getCoordinate3D(2)) ) {
	  sideList2->push(tempNode);
	}
	else if ( tempNode->hasCoordinate3D(currentNode->getCoordinate3D(1)) &&
		  tempNode->hasCoordinate3D(currentNode->getCoordinate3D(2)) ) {
	  sideList3->push(tempNode);
	}
	else {
	  printf("Error TriangleGraph::getTetrahedronGraph - neighboring vertex is not a neighbor\n");
	}
      }
      tempNode = currentNode->getNextVertex();
    }
    //printf("\n");
    //printf("  lengths: sideList1 %d, sideList2 %d, sideList3 %d\n",sideList1->getLength(),sideList2->getLength(),sideList3->getLength());
    

    // Proceed through sideList1.
    //printf("Proceed through sideList1.\n");
    while (sideList1->getLength() > 0) {
      side1 = sideList1->front();
      //printf("  side1: %d\n",side1->getId());
      // Get fourth coordinate of the possible tetrahedron.
      for (j=0; j<3; j++) {
	fourthCoord = side1->getCoordinate3D(j);
	if ( !currentNode->hasCoordinate3D(fourthCoord) ) {
	  break;
	}
      }
      // Search for opposite point in sideList2.
      side2 = sideList2->getFirst();
      while (side2 != 0) {
	//printf("    side2: %d\n",side2->getId());
	if (side2->hasCoordinate3D(fourthCoord)) {
	  // Search for opposite point in sideList3.
	  side3 = sideList3->getFirst();
	  while (side3 != 0) {
	    //printf("      side3: %d\n",side3->getId());
	    // if (this triangle is found) add tetrahedron to tetraList.
	    if (side3->hasCoordinate3D(fourthCoord)) {
	      //printf("        TETRAHEDRON (%d,%d,%d,%d)\n",currentNode->getId(),side1->getId(),side2->getId(),side3->getId());
	      TriangleGraphNode** tetrahedron = new TriangleGraphNode*[4];
	      tetrahedron[0] = currentNode;
	      tetrahedron[1] = side1;
	      tetrahedron[2] = side2;
	      tetrahedron[3] = side3;
	      tetraList->push(tetrahedron);
	    }
	    side3 = sideList3->getNext();
	  }
	}
	side2 = sideList2->getNext();
      }
    }
    
    delete sideList1;
    delete sideList2;
    delete sideList3;
  }

  // Add all the tetrahedra from tetraList to tGraph.
  //printf("Add all the tetrahedra from tetraList to tGraph.\n");
  //printf("  triangleList->getLength(): %d\n",triangleList->getLength());
  TetrahedronGraph* tetraGraph = new TetrahedronGraph(tetraList->getLength());
  TriangleGraphNode** tetrahedron = tetraList->front();
  int k = 0;
  while (tetrahedron != 0) {
    //printf("  k: %d\n",k);
    tetraGraph->addVertex(tetrahedron[0],tetrahedron[1],tetrahedron[2],tetrahedron[3]);
    delete tetrahedron;
    tetrahedron = tetraList->front();
    k++;
  }

  //tGraph->addEdges();

  //printf("<-TriangleGraph::getTetrahedronGraph\n");
  return tetraGraph;
}


/**
 * Identify pairs of triangles that share a side.
 * @param node1 TriangleGraphNode*: first node.
 * @param node2 TriangleGraphNode*: second node.
 * @return int: 0 if not a connected pair, 1 if connected.
 */
int TriangleGraph::arePaired(TriangleGraphNode* node1, TriangleGraphNode* node2) {



  return 0;
}


/**
 * Identify triples of triangles that share a point and share sides (pairwise).
 * @param node1 TriangleGraphNode*: first node.
 * @param node2 TriangleGraphNode*: second node.
 * @param node3 TriangleGraphNode*: third node.
 * @return int: 0 if not a connected triple, 1 if connected.
 */
int TriangleGraph::arePaired(TriangleGraphNode* node1,
			     TriangleGraphNode* node2,
			     TriangleGraphNode* node3) {



  return 0;
}


/**
 * Print a tcl tessellation file for the active vertices and edges of this graph.
 * @param outfile FILE*: tcl tessellation file in which to write the graph.
 * @param colorByValue int: whether to color the tessellation by value or not.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::printTessellationScript(FILE* outfile, int colorByValue) {

  fprintf(outfile,"\n");
  fprintf(outfile,"set mol top\n");
  fprintf(outfile,"\n");
  
  TriangleGraphNode* node1 = 0;
  Coordinate3D* coord1 = 0;
  Coordinate3D* coord2 = 0;
  Coordinate3D* coord3 = 0;

  if (colorByValue == 0) {
    int i=0;
    for (; i<vertexCount; i++) {
      node1 = nodes[i];
      if (node1->isActive() == 1) {
        coord1 = node1->getCoordinate3D(0);
        coord2 = node1->getCoordinate3D(1);
        coord3 = node1->getCoordinate3D(2);

	// XXX - debugging, triangle labels
	/*
	  Coordinate3D* coord5 = coord2->plus(coord3);
	  Coordinate3D* coord4 = coord1->plus(coord5);
	  coord4->setX(coord4->getX() / 3);
	  coord4->setY(coord4->getY() / 3);
	  coord4->setZ(coord4->getZ() / 3);
	  fprintf(outfile,"graphics $mol color 0\n");
	*/
	
	// Need two triangles so both sides are opaque.
	// First triangle.
        fprintf(outfile,"graphics $mol triangle");
        fprintf(outfile," {%3.2f %3.2f %3.2f}",
	        coord1->getX(),coord1->getY(),coord1->getZ());
        fprintf(outfile," {%3.2f %3.2f %3.2f}",
	        coord2->getX(),coord2->getY(),coord2->getZ());
        fprintf(outfile," {%3.2f %3.2f %3.2f}\n",
	        coord3->getX(),coord3->getY(),coord3->getZ());

	// XXX - debugging, triangle labels
	/*
	  fprintf(outfile,"graphics $mol color 8\n");
	  fprintf(outfile,"graphics $mol text {%3.2f %3.2f %3.2f} %d\n",
	  coord4->getX(),coord4->getY(),coord4->getZ(),i);
	  delete coord4;
	  delete coord5;
	*/

	// Second triangle.
	/*
	  fprintf(outfile,"graphics $mol triangle");
	  fprintf(outfile," {%3.2f %3.2f %3.2f}",
	  coord1->getX(),coord1->getY(),coord1->getZ());
	  fprintf(outfile," {%3.2f %3.2f %3.2f}",
	  coord3->getX(),coord3->getY(),coord3->getZ());
	  fprintf(outfile," {%3.2f %3.2f %3.2f}\n",
	  coord2->getX(),coord2->getY(),coord2->getZ());
	*/
      }
    }
  }
  // Color the graph by value.
  else {
    // Get min and max values for the entire set of vertices.
    // Scale vertex values from their location in [min,max] to [0,1]
    node1 = nodes[0];
    float* currentValue = new float[3];
    currentValue[0] = node1->getValue(0);
    float minValue = currentValue[0];
    float maxValue = currentValue[0];
    float meanValue = 0.0;
    int i = 1;
    int j = 1;
    for (; i<vertexCount; i++) {
      node1 = nodes[i];
      for (j=0; j<3; j++) {
        currentValue[0] = node1->getValue(j);
	meanValue += currentValue[0];
        if (currentValue[0] < minValue) {
          minValue = currentValue[0];
        }
        if (currentValue[0] > maxValue) {
          maxValue = currentValue[0];
        }
      }
    }
    meanValue /= (vertexCount * 3);
    //float offsetValue = maxValue - minValue;

    // Compute standard deviation.
    float diffFromMeanValue = 0.0;
    float sumOfMeanDiffSquared = 0.0;
    for (i=0; i<vertexCount; i++) {
      node1 = nodes[i];
      for (j=0; j<3; j++) {
        currentValue[0] = node1->getValue(j);
	diffFromMeanValue =  currentValue[0] - meanValue;
	sumOfMeanDiffSquared += diffFromMeanValue * diffFromMeanValue;
      }
    }
    float standardDeviation = sqrt(sumOfMeanDiffSquared / ((vertexCount*3)-1));
    float effectiveMaxValue = meanValue + 3 * standardDeviation;
    float offsetValue = effectiveMaxValue - minValue;

    fprintf(outfile,"\n");
    fprintf(outfile,"# min value: %4.3f, max value: %4.3f, effective max value: %4.3f\n",minValue,maxValue,effectiveMaxValue);
    fprintf(outfile,"# mean value: %4.3f, standard deviation: %4.3f\n",meanValue,standardDeviation);
    fprintf(outfile,"\n");
    float currentMeanValue = 0.0;  // Mean value of 3 triangle vertices.
    // Write tessellation file.
    for (i=0; i<vertexCount; i++) {
      node1 = nodes[i];
      if (node1->isActive() == 1) {
        for (j=0; j<3; j++) {
          // Normalize values.
          currentValue[j] = node1->getValue(j);
          currentValue[j] -= minValue;
          currentValue[j] /= offsetValue;
	  if (currentValue[j] > 1.0) {
	    currentValue[j] = 1.0;
          }
          // Specific for VMD colors, which are in the range [17,1040] with 1024 colors.
          currentValue[j] *= 1023;
          //currentValue[j] += 17;
          //if (currentValue[j] > 1040) {
          //  currentValue[j] = 1040;
          //}
        }
        currentMeanValue = (currentValue[0] + currentValue[1] + currentValue[2]) / 3;
        coord1 = node1->getCoordinate3D(0);
        coord2 = node1->getCoordinate3D(1);
        coord3 = node1->getCoordinate3D(2);
	// Need two triangles so both sides are opaque.
	// First triangle.
        fprintf(outfile,"graphics $mol color %4.0f\n",1040-currentMeanValue);
        fprintf(outfile,"graphics $mol triangle");
        fprintf(outfile," {%3.2f %3.2f %3.2f}",
	        coord1->getX(),coord1->getY(),coord1->getZ());
        fprintf(outfile," {%3.2f %3.2f %3.2f}",
	        coord2->getX(),coord2->getY(),coord2->getZ());
        fprintf(outfile," {%3.2f %3.2f %3.2f}\n",
	        coord3->getX(),coord3->getY(),coord3->getZ());
	// Second triangle.
	/*
        fprintf(outfile,"graphics $mol color %4.0f\n",1040-currentMeanValue);
        fprintf(outfile,"graphics $mol triangle");
        fprintf(outfile," {%3.2f %3.2f %3.2f}",
	        coord1->getX(),coord1->getY(),coord1->getZ());
        fprintf(outfile," {%3.2f %3.2f %3.2f}",
	        coord3->getX(),coord3->getY(),coord3->getZ());
        fprintf(outfile," {%3.2f %3.2f %3.2f}\n",
	        coord2->getX(),coord2->getY(),coord2->getZ());
	*/
      }
    }
    delete currentValue;
  }

  return 1;
}


/**
 * Print a tcl tessellation file for the active vertices and edges of this graph.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraph::printTessellationScript(FILE* outfile) {

  return printTessellationScript(outfile, 0);
}


/**
 * DEBUG PROC. Print out of node state.
 */
int TriangleGraph::print() {

  printf("TriangleGraph\n");
  printf("  vertexCount:       %d\n",vertexCount);
  printf("  edgeCount:         %d\n",edgeCount);
  printf("  activeVertexCount: %d\n",activeVertexCount);
  printf("  activeEdgeCount:   %d\n",activeEdgeCount);
  printf("  groupCount:        %d\n",groupCount);
  printf("  activeGroupCount:  %d\n",activeGroupCount);
  printf("  maxVertexCount:    %d\n",maxVertexCount);

  int i = 0;
  for (; i<vertexCount; i++) {
    printf("Node: %d\n", i);
    nodes[i]->print();
  }

  return 1;
}


int TriangleGraph::print2() {
  
  int i = 0;
  for (i=0; i<vertexCount; i++) {
    nodes[i]->print2();
  }

  return 1;
}
