
#ifndef TRIANGLEGRAPH_H
#define TRIANGLEGRAPH_H

//#include "baseGraph.h"
#include "baseVertexNode.h"
#include "coordinate3D.h"
#include "genericList.h"
#include "groupId.c"
#include "tetrahedronGraph.h"
#include "triangleEdgeTree.h"
#include "triangleGraphNode.h"

#include <stdio.h>


class TriangleGraph {

 public:
  /** Constructor. */
  TriangleGraph(int mVC);

  /** Destructor. */
  ~TriangleGraph();

  /**
   * Add nodes to the graph.
   * @param node1 BaseVertexNode*: first corner of triangle.
   * @param node2 BaseVertexNode*: second corner of triangle.
   * @param node3 BaseVertexNode*: third corner of triangle.
   * @return int: 0 on fail, 1 on success.
   */
  int addVertex(BaseVertexNode* node1, BaseVertexNode* node2, BaseVertexNode* node3);

  /**
   * Find and add edges to the graph by searching for triangles that share a side.
   * @return int: 0 on fail, 1 on success.
   */
  int addEdges();

  /**
   * Add an edge between two nodes in the graph.
   * @param id1 int: ID of first TriangleGraphNode
   * @param id2 int: ID of second TriangleGraphNode
   * @return int: 0 on fail, 1 on success.
   */
  int addEdge(int id1, int id2);

  /**
   * Get the surface area of the object.  INTERNAL triangles have no surface area, EXTERNAL triangles contribute one side, and DOUBLE triangles contribute both sides.
   * @return float: surface area.
   */
  float getSurfaceArea();

  /**
   * Activate all nodes in the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int activateAll();

  /**
   * Deactivate all nodes in the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivateAll();

  /**
   * Deactivate all INTERNAL type nodes in the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivateInternal();

  /**
   * Update or initially set the node IDs.
   * @return int: 0 on fail, 1 on success.
   */
  int refreshIds();

  /**
   * Update or initially set the nodes' group IDs.
   * @return int: 0 on fail, 1 on success.
   */
  int refreshGroupIds();

  /**
   * Create and return a TetrahedronGraph based on this BaseGraph.
   * @return TetrahedronGraph*: graph where a tetrahedron is a node
   */
  TetrahedronGraph* getTetrahedronGraph();

  /**
   * Identify pairs of triangles that share a side.
   * @param node1 TriangleGraphNode*: first node.
   * @param node2 TriangleGraphNode*: second node.
   * @return int: 0 if not a connected pair, 1 if connected.
   */
  int arePaired(TriangleGraphNode* node1, TriangleGraphNode* node2);

  /**
   * Identify triples of triangles that share a point and share sides (pairwise).
   * @param node1 TriangleGraphNode*: first node.
   * @param node2 TriangleGraphNode*: second node.
   * @param node3 TriangleGraphNode*: third node.
   * @return int: 0 if not a connected triple, 1 if connected.
   */
  int arePaired(TriangleGraphNode* node1, TriangleGraphNode* node2, TriangleGraphNode* node3);

  /**
   * Print a tcl tessellation file for the active vertices and edges of this graph.
   * @param outfile FILE*: tcl tessellation file in which to write the graph.
   * @param colorByValue int: whether to color the tessellation by value or not.
   * @return int: 0 on fail, 1 on success.
   */
  int printTessellationScript(FILE* outfile, int colorByValue);

  /**
   * Print a tcl tessellation file for the active vertices and edges of this graph.
   * @param outfile FILE*: tcl tessellation file in which to write the graph.
   * @return int: 0 on fail, 1 on success.
   */
  int printTessellationScript(FILE* outfile);


  /**
   * DEBUG PROC. Print out of node state.
   */
  int print();

  /**
   * DEBUG PROC. Print out of BaseVertexNodes' ids.
   */
  int print2();

 private:
  TriangleGraphNode** nodes;  // Array of nodes
  GenericList<TriangleGraphNode*>* groups;  // Connected subgraphs
  int vertexCount;
  int edgeCount;
  int activeVertexCount;
  int activeEdgeCount;
  int groupCount;
  int activeGroupCount;
  int maxVertexCount;
};

#endif
